@@ -1,17 +1,12 @@
+#include "utils-dispatch.h"
 #include "vctrs.h"
 #include "type-data-frame.h"
 #include "decl/type-data-frame-decl.h"
 
 bool is_data_frame(r_obj* x) {
-  if (r_typeof(x) != R_TYPE_list) {
-    return false;
-  }
-
-  enum vctrs_class_type type = class_type(x);
   return
-    type == VCTRS_CLASS_bare_data_frame ||
-    type == VCTRS_CLASS_bare_tibble ||
-    type == VCTRS_CLASS_data_frame;
+    r_typeof(x) == R_TYPE_list &&
+    class_type_is_data_frame(class_type(x));
 }
 
 bool is_native_df(r_obj* x) {
@@ -85,7 +80,7 @@ r_obj* ffi_new_data_frame(r_obj* args) {
 
   // Take names from `x` if `attrib` doesn't have any
   if (!has_names) {
-    r_obj* nms = vctrs_shared_empty_chr;
+    r_obj* nms = r_globals.empty_chr;
     if (r_length(out)) {
       nms = r_names(out);
     }
@@ -175,22 +170,22 @@ r_obj* ffi_data_frame(r_obj* x,
                       r_obj* size,
                       r_obj* name_repair,
                       r_obj* frame) {
-  struct r_lazy call = { .x = frame, .env = r_null };
+  struct r_lazy error_call = { .x = syms.dot_error_call, .env = frame };
 
   struct name_repair_opts name_repair_opts = new_name_repair_opts(name_repair,
-                                                                  vec_args.dot_name_repair,
+                                                                  lazy_args.dot_name_repair,
                                                                   false,
-                                                                  call);
+                                                                  error_call);
   KEEP(name_repair_opts.shelter);
 
   r_ssize c_size = 0;
   if (size == r_null) {
-    c_size = vec_check_size_common(x, 0, vec_args.empty, call);
+    c_size = vec_check_size_common(x, 0, vec_args.empty, error_call);
   } else {
-    c_size = vec_as_short_length(size, vec_args.dot_size, call);
+    c_size = vec_as_short_length(size, vec_args.dot_size, error_call);
   }
 
-  r_obj* out = data_frame(x, c_size, &name_repair_opts, call);
+  r_obj* out = data_frame(x, c_size, &name_repair_opts, error_call);
 
   FREE(1);
   return out;
@@ -200,8 +195,9 @@ static
 r_obj* data_frame(r_obj* x,
                   r_ssize size,
                   const struct name_repair_opts* p_name_repair_opts,
-                  struct r_lazy call) {
-  r_obj* out = KEEP(df_list(x, size, p_name_repair_opts, call));
+                  struct r_lazy error_call) {
+  const bool unpack = true;
+  r_obj* out = KEEP(df_list(x, size, unpack, p_name_repair_opts, error_call));
   out = new_data_frame(out, size);
   FREE(1);
   return out;
@@ -211,24 +207,27 @@ r_obj* data_frame(r_obj* x,
 // [[ register() ]]
 r_obj* ffi_df_list(r_obj* x,
                    r_obj* size,
+                   r_obj* unpack,
                    r_obj* name_repair,
                    r_obj* frame) {
-  struct r_lazy call = { .x = frame, .env = r_null };
+  struct r_lazy error_call = { .x = syms.dot_error_call, .env = frame };
 
   struct name_repair_opts name_repair_opts = new_name_repair_opts(name_repair,
-                                                                  vec_args.dot_name_repair,
+                                                                  lazy_args.dot_name_repair,
                                                                   false,
-                                                                  call);
+                                                                  error_call);
   KEEP(name_repair_opts.shelter);
 
   r_ssize c_size = 0;
   if (size == r_null) {
-    c_size = vec_check_size_common(x, 0, vec_args.empty, call);
+    c_size = vec_check_size_common(x, 0, vec_args.empty, error_call);
   } else {
-    c_size = vec_as_short_length(size, vec_args.dot_size, call);
+    c_size = vec_as_short_length(size, vec_args.dot_size, error_call);
   }
 
-  r_obj* out = df_list(x, c_size, &name_repair_opts, call);
+  const bool c_unpack = r_arg_as_bool(unpack, ".unpack");
+
+  r_obj* out = df_list(x, c_size, c_unpack, &name_repair_opts, error_call);
 
   FREE(1);
   return out;
@@ -237,13 +236,14 @@ r_obj* ffi_df_list(r_obj* x,
 static
 r_obj* df_list(r_obj* x,
                r_ssize size,
+               bool unpack,
                const struct name_repair_opts* p_name_repair_opts,
-               struct r_lazy call) {
+               struct r_lazy error_call) {
   if (r_typeof(x) != R_TYPE_list) {
     r_stop_internal("`x` must be a list.");
   }
 
-  x = KEEP(vec_check_recycle_common(x, size, vec_args.empty, call));
+  x = KEEP(vec_check_recycle_common(x, size, vec_args.empty, error_call));
 
   r_ssize n_cols = r_length(x);
 
@@ -255,7 +255,11 @@ r_obj* df_list(r_obj* x,
   }
 
   x = KEEP(df_list_drop_null(x));
-  x = KEEP(df_list_splice(x));
+
+  if (unpack) {
+    x = df_list_unpack(x);
+  }
+  KEEP(x);
 
   r_obj* names = KEEP(r_names(x));
   names = KEEP(vec_as_names(names, p_name_repair_opts));
@@ -303,16 +307,16 @@ r_obj* df_list_drop_null(r_obj* x) {
 }
 
 static
-r_obj* df_list_splice(r_obj* x) {
+r_obj* df_list_unpack(r_obj* x) {
   r_obj* names = KEEP(r_names(x));
   r_obj* const * p_names = r_chr_cbegin(names);
 
-  bool any_needs_splice = false;
+  bool any_needs_unpack = false;
   r_ssize n_cols = r_length(x);
   r_ssize i = 0;
 
   for (; i < n_cols; ++i) {
-    // Only splice unnamed data frames
+    // Only unpack unnamed data frames
     if (p_names[i] != strings_empty) {
       continue;
     }
@@ -320,27 +324,27 @@ r_obj* df_list_splice(r_obj* x) {
     r_obj* col = r_list_get(x, i);
 
     if (is_data_frame(col)) {
-      any_needs_splice = true;
+      any_needs_unpack = true;
       break;
     }
   }
 
-  if (!any_needs_splice) {
+  if (!any_needs_unpack) {
     FREE(1);
     return x;
   }
 
-  r_obj* splice = KEEP(r_new_logical(n_cols));
-  int* p_splice = LOGICAL(splice);
+  r_obj* unpack = KEEP(r_new_logical(n_cols));
+  int* p_unpack = LOGICAL(unpack);
 
   for (r_ssize j = 0; j < n_cols; ++j) {
-    p_splice[j] = 0;
+    p_unpack[j] = 0;
   }
 
   r_ssize width = i;
 
   for (; i < n_cols; ++i) {
-    // Only splice unnamed data frames
+    // Only unpack unnamed data frames
     if (p_names[i] != strings_empty) {
       ++width;
       continue;
@@ -350,7 +354,7 @@ r_obj* df_list_splice(r_obj* x) {
 
     if (is_data_frame(col)) {
       width += r_length(col);
-      p_splice[i] = 1;
+      p_unpack[i] = 1;
     } else {
       ++width;
     }
@@ -361,9 +365,9 @@ r_obj* df_list_splice(r_obj* x) {
 
   r_ssize loc = 0;
 
-  // Splice loop
+  // Unpack loop
   for (r_ssize i = 0; i < n_cols; ++i) {
-    if (!p_splice[i]) {
+    if (!p_unpack[i]) {
       r_list_poke(out, loc, r_list_get(x, i));
       r_chr_poke(out_names, loc, p_names[i]);
       ++loc;
@@ -404,12 +408,12 @@ r_obj* df_list_splice(r_obj* x) {
 enum rownames_type rownames_type(r_obj* x) {
   switch (r_typeof(x)) {
   case R_TYPE_character:
-    return ROWNAMES_IDENTIFIERS;
+    return ROWNAMES_TYPE_identifiers;
   case R_TYPE_integer:
     if (r_length(x) == 2 && r_int_begin(x)[0] == r_globals.na_int) {
-      return ROWNAMES_AUTOMATIC_COMPACT;
+      return ROWNAMES_TYPE_automatic_compact;
     } else {
-      return ROWNAMES_AUTOMATIC;
+      return ROWNAMES_TYPE_automatic;
     }
   default:
     r_stop_internal("Unexpected type `%s`.", Rf_type2char(r_typeof(x)));
@@ -424,10 +428,10 @@ r_ssize compact_rownames_length(r_obj* x) {
 // [[ include("type-data-frame.h") ]]
 r_ssize rownames_size(r_obj* rn) {
   switch (rownames_type(rn)) {
-  case ROWNAMES_IDENTIFIERS:
-  case ROWNAMES_AUTOMATIC:
+  case ROWNAMES_TYPE_identifiers:
+  case ROWNAMES_TYPE_automatic:
     return r_length(rn);
-  case ROWNAMES_AUTOMATIC_COMPACT:
+  case ROWNAMES_TYPE_automatic_compact:
     return compact_rownames_length(rn);
   }
 
@@ -448,7 +452,7 @@ void init_tibble(r_obj* x, r_ssize n) {
 static
 void init_bare_data_frame(r_obj* x, r_ssize n) {
   if (r_length(x) == 0) {
-    r_attrib_poke(x, r_syms.names, vctrs_shared_empty_chr);
+    r_attrib_poke(x, r_syms.names, r_globals.empty_chr);
   }
 
   init_compact_rownames(x, n);
@@ -464,7 +468,7 @@ void init_compact_rownames(r_obj* x, r_ssize n) {
 static
 r_obj* new_compact_rownames(r_ssize n) {
   if (n <= 0) {
-    return vctrs_shared_empty_int;
+    return r_globals.empty_int;
   }
 
   r_obj* out = r_alloc_integer(2);
@@ -735,7 +739,7 @@ r_obj* df_cast_match(const struct cast_opts* opts,
       // `base_c_invoke()`.
       if (opts->fallback.s3 && vec_is_common_class_fallback(to_col)) {
         KEEP(col);
-        r_attrib_poke(col, r_sym("vctrs:::unspecified"), vctrs_shared_true);
+        r_attrib_poke(col, r_sym("vctrs:::unspecified"), r_true);
         FREE(1);
       }
     } else {
